Date: 2022-10-01T06:40:16+00:00

Actual source code: test10.c
slepc-3.18.0 2022-10-01
Report Typos and Errors

      1: /*
      2:    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      3:    SLEPc - Scalable Library for Eigenvalue Problem Computations
      4:    Copyright (c) 2002-, Universitat Politecnica de Valencia, Spain

      6:    This file is part of SLEPc.
      7:    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
      8:    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
      9: */

     11: static char help[] = "Lanczos SVD. Also illustrates the use of SVDSetBV().\n\n"
     12:   "The command line options are:\n"
     13:   "  -m <m>, where <m> = matrix rows.\n"
     14:   "  -n <n>, where <n> = matrix columns (defaults to m+2).\n\n";

     16: #include <slepcsvd.h>

     18: /*
     19:    This example computes the singular values of a rectangular bidiagonal matrix

     21:               |  1  2                     |
     22:               |     1  2                  |
     23:               |        1  2               |
     24:           A = |          .  .             |
     25:               |             .  .          |
     26:               |                1  2       |
     27:               |                   1  2    |
     28:  */

     30: int main(int argc,char **argv)
     31: {
     32:   Mat            A;
     33:   SVD            svd;
     34:   PetscInt       m=20,n,Istart,Iend,i,k=6,col[2];
     35:   PetscScalar    value[] = { 1, 2 };
     36:   PetscBool      flg,oneside=PETSC_FALSE;
     37:   const char     *prefix;
     38:   BV             U,V;
     39:   Vec            u,v;

     42:   SlepcInitialize(&argc,&argv,(char*)0,help);
     43:   PetscOptionsGetInt(NULL,NULL,"-m",&m,NULL);
     44:   PetscOptionsGetInt(NULL,NULL,"-n",&n,&flg);
     45:   if (!flg) n=m+2;
     46:   PetscOptionsGetInt(NULL,NULL,"-k",&k,NULL);
     47:   PetscPrintf(PETSC_COMM_WORLD,"\nRectangular bidiagonal matrix, m=%" PetscInt_FMT " n=%" PetscInt_FMT "\n\n",m,n);

     49:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     50:         Generate the matrix
     51:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

     53:   MatCreate(PETSC_COMM_WORLD,&A);
     54:   MatSetSizes(A,PETSC_DECIDE,PETSC_DECIDE,m,n);
     55:   MatSetFromOptions(A);
     56:   MatSetUp(A);
     57:   MatGetOwnershipRange(A,&Istart,&Iend);
     58:   for (i=Istart;i<Iend;i++) {
     59:     col[0]=i; col[1]=i+1;
     60:     if (i<n-1) MatSetValues(A,1,&i,2,col,value,INSERT_VALUES);
     61:     else if (i==n-1) MatSetValue(A,i,col[0],value[0],INSERT_VALUES);
     62:   }
     63:   MatAssemblyBegin(A,MAT_FINAL_ASSEMBLY);
     64:   MatAssemblyEnd(A,MAT_FINAL_ASSEMBLY);
     65:   MatCreateVecs(A,&v,&u);

     67:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     68:          Create standalone BV objects to illustrate use of SVDSetBV()
     69:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

     71:   BVCreate(PETSC_COMM_WORLD,&U);
     72:   PetscObjectSetName((PetscObject)U,"U");
     73:   BVSetSizesFromVec(U,u,k);
     74:   BVSetFromOptions(U);
     75:   BVCreate(PETSC_COMM_WORLD,&V);
     76:   PetscObjectSetName((PetscObject)V,"V");
     77:   BVSetSizesFromVec(V,v,k);
     78:   BVSetFromOptions(V);

     80:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
     81:         Compute singular values
     82:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

     84:   SVDCreate(PETSC_COMM_WORLD,&svd);
     85:   SVDSetBV(svd,V,U);
     86:   SVDSetOptionsPrefix(svd,"check_");
     87:   SVDAppendOptionsPrefix(svd,"myprefix_");
     88:   SVDGetOptionsPrefix(svd,&prefix);
     89:   PetscPrintf(PETSC_COMM_WORLD,"SVD prefix is currently: %s\n\n",prefix);
     90:   PetscObjectSetName((PetscObject)svd,"SVD_solver");

     92:   SVDSetOperators(svd,A,NULL);
     93:   SVDSetType(svd,SVDLANCZOS);
     94:   SVDSetFromOptions(svd);

     96:   PetscObjectTypeCompare((PetscObject)svd,SVDLANCZOS,&flg);
     97:   if (flg) {
     98:     SVDLanczosGetOneSide(svd,&oneside);
     99:     PetscPrintf(PETSC_COMM_WORLD,"Running Lanczos %s\n\n",oneside?"(onesided)":"");
    100:   }
    101:   PetscObjectTypeCompare((PetscObject)svd,SVDTRLANCZOS,&flg);
    102:   if (flg) {
    103:     SVDTRLanczosGetOneSide(svd,&oneside);
    104:     PetscPrintf(PETSC_COMM_WORLD,"Running thick-restart Lanczos %s\n\n",oneside?"(onesided)":"");
    105:   }

    107:   SVDSolve(svd);

    109:   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    110:                     Display solution and clean up
    111:      - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
    112:   SVDErrorView(svd,SVD_ERROR_RELATIVE,PETSC_VIEWER_STDOUT_WORLD);
    113:   BVDestroy(&U);
    114:   BVDestroy(&V);
    115:   VecDestroy(&u);
    116:   VecDestroy(&v);
    117:   SVDDestroy(&svd);
    118:   MatDestroy(&A);
    119:   SlepcFinalize();
    120:   return 0;
    121: }

    123: /*TEST

    125:    testset:
    126:       args: -check_myprefix_svd_nsv 3
    127:       requires: double
    128:       test:
    129:          suffix: 1
    130:          args: -check_myprefix_svd_view_vectors ::ascii_info
    131:       test:
    132:          suffix: 2
    133:          args: -check_myprefix_svd_type trlanczos -check_myprefix_svd_monitor -check_myprefix_svd_view_values ::ascii_matlab
    134:          filter: sed -e "s/[0-9]\.[0-9]*e[+-]\([0-9]*\)/removed/g"
    135:       test:
    136:          suffix: 3
    137:          args: -m 22 -n 20

    139: TEST*/